<?php

class SOPGRefundMerchantClient
{
	private $transaction = array();
	public $error = array( 'msg' => null, 'code' => null );
	private $client = false;
	private $productive;
	private $refundResult;
	private $url = array( 'test' => 'https://soatest.paysafecard.com/psc/services/PscService?wsdl', 'productive' =>
			'https://soa.paysafecard.com/psc/services/PscService?wsdl' );
	private $wsdl = array();
	public function __construct( $username, $password, $productive = false )
	{
		$this->transaction['username'] = $username;
		$this->transaction['password'] = $password;
		$this->productive = $productive;
	}
	public function setTransaction( $amount, $currency,$mtid = '', $rtid = '', $utcOffset = '+01:00' )
	{
		$this->transaction['rtid'] = ( $rtid ) ? trim($rtid) : $this->generateRtid();
        $this->transaction['mtid'] = trim($mtid);
		$this->transaction['amount'] = number_format(str_replace( ',', '.', $amount ),2,'.','');
		$this->transaction['currency'] = strtoupper( $currency );
		$this->transaction['utcOffset'] = $utcOffset;
	}
	public function setCustomer( $merchantClientId, $customerId, $customerIdType = 'EMAIL' )
	{
		$this->transaction['customerIdType'] = $customerIdType;
		$this->transaction['customerId'] = $customerId;
		$this->transaction['merchantClientId'] = $merchantClientId;
	}
	public function refund( $validationOnly = true )
	{
		$this->transaction['validationOnly'] = $validationOnly;
		$this->client = !$this->client?$this->connect():$this->client;
		$result = $this->client->refund( $this->transaction );
		return $this->refundResult( $result );
	}
	private function refundResult( $result )
	{
		$this->refundResult = array(
			'status' => ( $result->refundRequestReturnType->errorCode == 0 && $result->refundRequestReturnType->resultCode == 0 ) ? true : false,
			'errorMsg' => ( $result->refundRequestReturnType->errorCode == 0 && $result->refundRequestReturnType->resultCode == 0 ) ? false : $this->setError
				( $result->refundRequestReturnType->errorCode,$result->refundRequestReturnType->errorCodeDescription),
			'rtid' => $result->refundRequestReturnType->rtid,
            'mtid' => $result->refundRequestReturnType->mtid,
            'validationOnly' => $result->refundRequestReturnType->validationOnly,
			'transactionDetails' => array(
				'requestedCurrency' => $result->refundRequestReturnType->requestedCurrency,
				'requestedAmount' => $result->refundRequestReturnType->requestedAmount,
				'refundedInCurrency' => ( isset( $result->refundRequestReturnType->refundedInCurrency ) ) ? $result->refundRequestReturnType->refundedInCurrency : null, 
				'refundedAmount' => ( isset( $result->refundRequestReturnType->refundedAmount ) ) ? $result->refundRequestReturnType->refundedAmount : null ), 
			'codes' => array(
				'resultCode' => $result->refundRequestReturnType->resultCode,
				'errorCode' => $result->refundRequestReturnType->errorCode,
				'errorCodeDescription' => $result->refundRequestReturnType->errorCodeDescription ),
			);
		return $this->refundResult['status'];
	}
	private function connect()
	{
		$this->soapInit();
		try
		{
			return new SoapClient( ( $this->productive ) ? $this->url['productive'] : $this->url['test'], $this->wsdl );
		}
		catch ( exception $e )
		{
			echo 'Exception: ', $e->getMessage(), "\n";
		}
	}
	private function generateRtid()
	{
		return 'refundTest_' . date( 'Ymd-His' ) . '_' . rand( 1, 9999 );
	}
	private function soapInit()
	{
		if ( !class_exists( "SOAPClient" ) )
		{
			die( 'ERROR: SOAPClient isn´t enabled' );
		}
		$this->wsdl = array(
			'encoding' => 'utf-8',
			'connection_timeout' => 15,
			'features' => SOAP_SINGLE_ELEMENT_ARRAYS,
			'compression' => SOAP_COMPRESSION_GZIP );
		if ( !$this->productive )
		{
			ini_set( "soap.wsdl_cache_enabled", '0' );
			ini_set( "soap.wsdl_cache_ttl", '0' );
			$wsdl = array(
				'user_agent' => 'paysafecard refund class(testmode) v1.0',
				'trace' => true,
				'exceptions' => true,
				'cache_wsdl' => WSDL_CACHE_NONE );
		}
		else
		{
			$wsdl = array(
				'user_agent' => 'paysafecard refund class v1.0',
				'trace' => false,
				'exceptions' => false );
		}
		$this->wsdl = array_merge( $this->wsdl, $wsdl );
	}
	public function getDetails( $dump = false )
	{
		$debug = array(
			'transaction' => $this->transaction,
			'refundResult' => $this->refundResult,
			'soap' => $this->wsdl );
		if ( $dump )
		{
			echo '<pre>';
			print_r( $debug );
			echo '</pre>';
		}
		return $debug;
	}
	private function setError( $errorCode,$errorCodeDesc=false )
	{
		switch ( $errorCode )
		{
			case '3184':
                $this->error['msg'] = 'Original payment transaction does not exist.';
				break;
			case '3162':
				$this->error['msg'] = 'E-mail address is not registered with mypaysafecard.';
				break;
			case '3182':
				$this->error['msg'] = 'Merchantclient-ID is a mandatory parameter.';
				break;
            case '3181':
				$this->error['msg'] = 'Merchantlcient-ID is not matching with original transaction';
				break;
			default:
				$this->error['msg'] = 'A technical error occurred. ErrorCode: '.$errorCode.' - '.$errorCodeDesc.'. Please contact your support department.';
		}
		$this->error['code'] = $errorCode;
		return $this->error['msg'];
	}
}
